﻿using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Media;
using ARCompass.Behaviors;
using Phone7.Fx.Preview;
using Wp7nl.Utilities;

namespace HeadsUpCompass.Behaviors
{
  public class CompassDirectionDisplayBehavior : Behavior<FrameworkElement>
  {
    private FrameworkElement elementToAnimate;

    private FrameworkElement displayCanvas;

    private LocationCalcutator calculator;

    #region Setup
    protected override void OnAttached()
    {
      base.OnAttached();
      AssociatedObject.Loaded += AssociatedObjectLoaded;
      AssociatedObject.Unloaded += AssociatedObjectUnloaded;
      
    }

    void AssociatedObjectLoaded(object sender, RoutedEventArgs e)
    {
      calculator = new LocationCalcutator {DisplayCompassDirection = DisplayCompassDirection};
      elementToAnimate = AssociatedObject.GetElementToAnimate();
      if (!(elementToAnimate.RenderTransform is CompositeTransform))
      {
        elementToAnimate.RenderTransform = new CompositeTransform();
      }
      displayCanvas = elementToAnimate.GetVisualParent();
      if (displayCanvas != null)
      {
        displayCanvas.SizeChanged += DisplayCanvasSizeChanged;
        UpdateCalculator();
      }
    }
    #endregion

    #region Cleanup
    private bool isCleanedUp;

    private void Cleanup()
    {
      if (!isCleanedUp)
      {
        isCleanedUp = true;
        AssociatedObject.Loaded -= AssociatedObjectLoaded;
        AssociatedObject.Unloaded -= AssociatedObjectUnloaded;
      }
    }

    protected override void OnDetaching()
    {
      Cleanup();
      base.OnDetaching();
    }

    void AssociatedObjectUnloaded(object sender, RoutedEventArgs e)
    {
      Cleanup();
    }
    #endregion

    void DisplayCanvasSizeChanged(object sender, SizeChangedEventArgs e)
    {
      UpdateCalculator();
    }

    private void UpdateCalculator()
    {
      calculator.CanvasHeight = displayCanvas.ActualHeight;
      calculator.CanvasWidth = displayCanvas.ActualWidth;
      calculator.SetHorizontalPixels(6 * calculator.CanvasWidth);
      UpdateScreenLocation();
    }

    void UpdateScreenLocation()
    {
      var translationPoint = calculator.CalculateScreenPosition(CurrentCompassDirection);
      if (calculator.IsVisible(translationPoint))
      {
        elementToAnimate.SetTranslatePoint(calculator.CalculateScreenPosition(CurrentCompassDirection));
        elementToAnimate.Visibility = Visibility.Visible;
      }
      else
      {
        elementToAnimate.Visibility = Visibility.Collapsed;
      }
    }


    #region DisplayCompassDirection
    public const string DisplayCompassDirectionPropertyName = "DisplayCompassDirection";

    public int DisplayCompassDirection
    {
      get { return (int)GetValue(DisplayCompassDirectionProperty); }
      set { SetValue(DisplayCompassDirectionProperty, value); }
    }

    public static readonly DependencyProperty DisplayCompassDirectionProperty = DependencyProperty.Register(
        DisplayCompassDirectionPropertyName,
        typeof(int),
        typeof(CompassDirectionDisplayBehavior),
        new PropertyMetadata(0, DisplayCompassDirectionChanged));

    public static void DisplayCompassDirectionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {

    }
    #endregion

    #region CurrentCompassDirection
    public const string CurrentCompassDirectionPropertyName = "CurrentCompassDirection";

    public int CurrentCompassDirection
    {
      get { return (int)GetValue(CurrentCompassDirectionProperty); }
      set { SetValue(CurrentCompassDirectionProperty, value); }
    }

    public static readonly DependencyProperty CurrentCompassDirectionProperty = DependencyProperty.Register(
        CurrentCompassDirectionPropertyName,
        typeof(int),
        typeof(CompassDirectionDisplayBehavior),
        new PropertyMetadata(0, CurrentCompassDirectionChanged));

    public static void CurrentCompassDirectionChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
      var behavior = d as CompassDirectionDisplayBehavior;
      if (behavior != null)
      {
        behavior.UpdateScreenLocation();
      }
    }
    #endregion
  }
}
